=== modified file 'MANIFEST.in'
--- MANIFEST.in	2011-03-10 14:22:12 +0000
+++ MANIFEST.in	2012-02-02 04:58:46 +0000
@@ -1,2 +1,6 @@
-include README.txt NEWS
+include NEWS
+include README.txt
+include apache-loggerhead.conf
+include bazaar.conf
+include loggerhead.conf.example
 recursive-include loggerhead/static *

=== modified file 'NEWS'
--- NEWS	2011-11-08 16:29:37 +0000
+++ NEWS	2012-02-02 07:42:24 +0000
@@ -1,8 +1,8 @@
 What's changed in loggerhead?
 =============================
 
-1.19 [???]
-----------------
+NEXT
+----
 
     - Add ``bzr load-test-loggerhead`` as a way to make sure loggerhead can
       handle concurrent requests, etc. Scripts can be written that spawn
@@ -49,6 +49,12 @@
       marked as private and a notification ribbon to be displayed via
       javascript on the view pages. (JC Sackett, #823471)
 
+    - Add a script and documentation for running under mod_wsgi.
+      (Stuart Colville, Toshio Kuratomi)
+
+    - Make tz calculations consistent and use UTC in the UI everywhere we show
+      a precise timestamp. (Robert Collins, #594591)
+
 
 1.18.1 [24Mar2011]
 ------------------
@@ -94,7 +100,6 @@
     - Fix bad redirect when visiting "/download" or "/download/".
       (Matt Nordhoff, #247992)
 
-
 1.17 [20Aug2009]
 ----------------
 

=== added file 'apache-loggerhead.conf'
--- apache-loggerhead.conf	1970-01-01 00:00:00 +0000
+++ apache-loggerhead.conf	2010-04-18 17:55:33 +0000
@@ -0,0 +1,19 @@
+### If you receive MemoryError tracebacks setting up loggerhead under mod_wsgi,
+### read /usr/share/doc/loggerhead*/README for help
+Alias /bzr/static /usr/share/loggerhead/static
+RewriteEngine On
+RewriteRule ^/bzr$ /bzr/ [R]
+
+WSGIDaemonProcess loggerhead user=apache group=apache maximum-requests=1000 display-name=loggerhead processes=4 threads=1
+WSGISocketPrefix run/wsgi
+WSGIRestrictStdout On
+WSGIRestrictSignal Off
+
+WSGIScriptAlias /bzr /usr/bin/loggerhead.wsgi
+
+<Location /bzr>
+    WSGIProcessGroup loggerhead
+    Order deny,allow
+    Allow from all
+</Location>
+

=== added file 'bazaar.conf'
--- bazaar.conf	1970-01-01 00:00:00 +0000
+++ bazaar.conf	2010-04-18 14:41:23 +0000
@@ -0,0 +1,9 @@
+# directory to serve bzr branches from
+# Non-bzr directories under this path will also be visible in loggerhead
+#http_root_dir = '/var/www/bzr'
+
+# The url prefix for the bzr branches.
+http_user_prefix = '/bzr'
+
+# Directory to put cache files in
+http_sql_dir = '/var/cache/loggerhead'

=== modified file 'docs/index.rst'
--- docs/index.rst	2010-05-14 07:33:29 +0000
+++ docs/index.rst	2012-02-02 04:58:46 +0000
@@ -149,6 +149,61 @@
 some path into the site, you'll need to specify it using
 ``--prefix=/some_path``.
 
+Serving Loggerhead with mod_wsgi
+--------------------------------
+
+A second method for using Loggerhead with apache is to have apache itself
+execute Loggerhead via mod_wsgi.  You need to add configuration for apache and
+for bazaar to make this work.  Example config files are in the Loggerhead doc
+directory as apache-loggerhead.conf and bazaar.conf.  You can copy them into
+place and use them as a starting point following these directions:
+
+1) Install mod_wsgi.  On Ubuntu and other Debian derived distros::
+
+    sudo apt-get install libapache2-mod-wsgi
+
+   On Fedora-derived distros::
+
+    su -c yum install mod_wsgi
+
+2) Copy the bazaar.conf file where apache will find it (May be done for you if
+   you installed Loggerhead from a distribution package)::
+
+    # install -d -o apache -g apache -m 0755 /etc/loggerhead
+    # cp -p /usr/share/doc/loggerhead*/bazaar.conf /etc/loggerhead/
+    # ln -s /etc/loggerhead /var/www/.bazaar
+
+3) Create the cache directory (May be done for you if you installed Loggerhead
+   from a distribution package)::
+
+    # install -d -o apache -g apache -m 0700 /var/cache/loggerhead/
+
+4) Edit /etc/loggerhead/bazaar.conf.  You need to set http_root_dir to the filesystem
+   path that you will find your bzr branches under.  Note that normal
+   directories under that path will also be visible in Loggerhead.
+
+5) Install the apache conf file::
+
+     # cp -p /usr/share/doc/loggerhead*/apache-loggerhead.conf /etc/httpd/conf.d/loggerhead.conf
+
+6) Edit /etc/httpd/conf.d/loggerhead.conf to point to the url you desire to
+   serve Loggerhead on.  This should match with the setting for
+   http_user_prefix in bazaar.conf
+
+7) Restart apache and you should be able to start browsing
+
+.. note:: If you have SELinux enabled on your system you may need to allow
+   apache to execute files in temporary directories.  You will get a
+   MemoryError traceback from python if this is the case.  This is because of
+   the way that python ctypes interacts with libffi.  To rectify this, you may
+   have to do several things, such as mounting tmpdirs so programs can be
+   executed on them and setting this SELinux boolean::
+
+       setsebool httpd_tmp_exec on
+
+   This bug has information about how python and/or Linux distros might solve
+   this issue permanently and links to bugs which diagnose the root cause.
+   https://bugzilla.redhat.com/show_bug.cgi?id=582009
 
 Search
 ------

=== added file 'loggerhead.wsgi'
--- loggerhead.wsgi	1970-01-01 00:00:00 +0000
+++ loggerhead.wsgi	2010-04-18 17:58:04 +0000
@@ -0,0 +1,50 @@
+#!/usr/bin/python -tt
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import sys
+import os
+import pwd
+sys.path.insert(0, os.path.dirname(__file__))
+
+from paste.httpexceptions import HTTPExceptionHandler
+from loggerhead.apps.transport import BranchesFromTransportRoot
+from loggerhead.apps.error import ErrorHandlerApp
+from loggerhead.config import LoggerheadConfig
+from bzrlib import config as bzrconfig
+from paste.deploy.config import PrefixMiddleware
+from bzrlib.plugin import load_plugins
+
+class NotConfiguredError(Exception):
+    pass
+
+
+load_plugins()
+config = LoggerheadConfig()
+prefix = config.get_option('user_prefix') or ''
+# Note we could use LoggerheadConfig here if it didn't fail when a
+# config option is not also a commandline option
+root_dir = bzrconfig.GlobalConfig().get_user_option('http_root_dir')
+if not root_dir:
+    raise NotConfiguredError('You must have a ~/.bazaar/bazaar.conf file for'
+            ' %(user)s with http_root_dir set to the base directory you want'
+            ' to serve bazaar repositories from' %
+            {'user': pwd.getpwuid(os.geteuid()).pw_name})
+prefix = prefix.encode('utf-8', 'ignore')
+root_dir = root_dir.encode('utf-8', 'ignore')
+app = BranchesFromTransportRoot(root_dir, config)
+app = PrefixMiddleware(app, prefix=prefix)
+app = HTTPExceptionHandler(app)
+application = ErrorHandlerApp(app)

=== modified file 'loggerhead/controllers/directory_ui.py'
--- loggerhead/controllers/directory_ui.py	2009-10-17 06:35:33 +0000
+++ loggerhead/controllers/directory_ui.py	2012-02-02 07:42:24 +0000
@@ -35,7 +35,7 @@
         if branch is not None:
             # If a branch is empty, bzr raises an exception when trying this
             try:
-                self.last_change = datetime.datetime.fromtimestamp(
+                self.last_change = datetime.datetime.utcfromtimestamp(
                     branch.repository.get_revision(
                         branch.last_revision()).timestamp)
             except:

=== modified file 'loggerhead/history.py'
--- loggerhead/history.py	2012-01-08 22:04:51 +0000
+++ loggerhead/history.py	2012-01-23 21:10:46 +0000
@@ -136,22 +136,19 @@
                 new_revision=self.revid(self.new_inv, file_id)))
         if versioned == 'added':
             self.added.append(util.Container(
-                filename=rich_filename(paths[1], kind),
-                file_id=file_id, kind=kind[1]))
+                filename=rich_filename(paths[1], kind), kind=kind[1]))
         elif versioned == 'removed':
             self.removed.append(util.Container(
-                filename=rich_filename(paths[0], kind),
-                file_id=file_id, kind=kind[0]))
+                filename=rich_filename(paths[0], kind), kind=kind[0]))
         elif renamed:
             self.renamed.append(util.Container(
                 old_filename=rich_filename(paths[0], kind[0]),
                 new_filename=rich_filename(paths[1], kind[1]),
-                file_id=file_id,
-                text_modified=modified == 'modified'))
+                text_modified=modified == 'modified', exe_change=exe_change))
         else:
             self.modified.append(util.Container(
                 filename=rich_filename(paths[1], kind),
-                file_id=file_id))
+                text_modified=modified == 'modified', exe_change=exe_change))
 
 # The lru_cache is not thread-safe, so we need a lock around it for
 # all threads.

=== modified file 'loggerhead/templates/breadcrumbs.pt'
--- loggerhead/templates/breadcrumbs.pt	2008-09-30 02:04:14 +0000
+++ loggerhead/templates/breadcrumbs.pt	2012-01-08 21:43:42 +0000
@@ -10,7 +10,7 @@
 
 <!-- Branch breadcrumbs (for the path within the branch) -->
 <span metal:define-macro="branch" class="breadcrumb">
-    <tal:block repeat="crumb branch_breadcrumbs">/<a tal:attributes="href python:url([crumb['suffix'], change.revno], file_id=crumb['file_id'])" tal:content="crumb/dir_name" tal:condition="not:repeat/crumb/end" /><span tal:replace="crumb/dir_name" tal:condition="repeat/crumb/end" /></tal:block>
+    <tal:block repeat="crumb branch_breadcrumbs">/<a tal:attributes="href python:url([crumb['suffix'], change.revno, crumb['path']])" tal:content="crumb/dir_name" tal:condition="not:repeat/crumb/end" /><span tal:replace="crumb/dir_name" tal:condition="repeat/crumb/end" /></tal:block>
 </span>
 
 <!-- Directory breadcrumbs (for the path leading up to the branch) -->

=== modified file 'loggerhead/templates/changelog.pt'
--- loggerhead/templates/changelog.pt	2011-02-23 03:01:15 +0000
+++ loggerhead/templates/changelog.pt	2012-02-02 07:42:24 +0000
@@ -129,8 +129,8 @@
             <td tal:condition="show_tag_col" tal:content="string:${entry/tags}"
                 class="tagcell"></td>
             <td class="date">
-              <span tal:attributes="title python:util.date_time(entry.date)"
-                    tal:content="python:util._approximatedate(entry.date)"></span>
+              <span tal:attributes="title python:util.date_time(entry.utc_date)"
+                    tal:content="python:util._approximatedate(entry.utc_date)"></span>
             </td>
             <td class="diffr"><a tal:attributes="title python:'Show diff at revision '+entry.revno;
                                                  href python:url(['/revision', entry.revno], clear=1)">

=== modified file 'loggerhead/templates/inventory.pt'
--- loggerhead/templates/inventory.pt	2011-04-22 18:18:02 +0000
+++ loggerhead/templates/inventory.pt	2012-02-02 07:42:24 +0000
@@ -84,7 +84,7 @@
                                                      title string:Show revision ${file/change/revno}"
                                      tal:content="file/change/revno"></a>
                  </td>
-                 <td class="date" tal:content="python:util.date_time(file.change.date)"></td>
+                 <td class="date" tal:content="python:util.date_time(file.change.utc_date)"></td>
                  <td class="timedate2"></td>
                  <td class="expcell"><a tal:attributes="href python:url(['/changes'], start_revid=start_revid, filter_file_id=file.file_id);
                                                      title string:Show revision ${file/change/revno}">
@@ -107,7 +107,7 @@
                                                      title string:Show revision ${file/change/revno}"
                                      tal:content="file/change/revno"></a>
                  </td>
-                 <td class="date" tal:content="python:util.date_time(file.change.date)"></td>
+                 <td class="date" tal:content="python:util.date_time(file.change.utc_date)"></td>
                  <td class="timedate2">.</td>
                  <td class="expcell"><a tal:attributes="href python:url(['/changes'], start_revid=start_revid, filter_file_id=file.file_id);
                                                         title string:Show revision ${file/change/revno}">
@@ -136,7 +136,7 @@
                                                   title string:Show revision ${file/change/revno}"
                                   tal:content="file/change/revno"></a>
               </td>
-              <td class="date" tal:content="python:util.date_time(file.change.date)"></td>
+              <td class="date" tal:content="python:util.date_time(file.change.utc_date)"></td>
               <td class="timedate2" tal:content="python:util.human_size(file.size)"></td>
               <td class="expcell"><a tal:attributes="href python:url(['/view', revno_url, file.absolutepath]);
                                                      title string:View ${file/filename}">

=== modified file 'loggerhead/templates/revisionfilechanges.pt'
--- loggerhead/templates/revisionfilechanges.pt	2009-03-19 00:05:41 +0000
+++ loggerhead/templates/revisionfilechanges.pt	2012-01-08 23:01:10 +0000
@@ -36,6 +36,7 @@
       <tal:old-name condition="not:renamed/text_modified" content="renamed/new_filename" class="filename">
         new_filename
       </tal:old-name>
+      <tal:prop-change condition="renamed/exe_change" content="string:*" />
     </li>
   </ul>
 
@@ -44,7 +45,9 @@
       <li class="desc">files modified:</li>
     </tal:block>
     <li class="files">
-      <tal:revision content="structure python:file_link(item.filename)" />
+      <tal:revision condition="item/text_modified" content="structure python:file_link(item.filename)" />
+      <tal:revision condition="not:item/text_modified" content="item/filename" />
+      <tal:prop-change condition="item/exe_change" content="string:*" />
     </li>
   </ul>
 </tal:block>

=== modified file 'loggerhead/templates/revisioninfo.pt'
--- loggerhead/templates/revisioninfo.pt	2011-03-16 12:37:17 +0000
+++ loggerhead/templates/revisioninfo.pt	2012-02-02 07:42:24 +0000
@@ -11,7 +11,7 @@
             </li>
             <li class="timer">
                 <strong>Date:</strong>
-                <span tal:content="python:util.date_time(change.date)"></span>
+                <span tal:content="python:util.date_time(change.utc_date)"></span>
             </li>
             <li class="mfrom" tal:condition="python:change.parents[1:]">
                 <strong>mfrom:</strong>

=== modified file 'loggerhead/templates/view.pt'
--- loggerhead/templates/view.pt	2011-04-22 17:22:26 +0000
+++ loggerhead/templates/view.pt	2012-02-02 07:42:24 +0000
@@ -79,7 +79,7 @@
                   class="viewRev">
                 <a tal:content="python:'%s' % (anno.change.revno,)"
                    tal:attributes="href python:url(['/revision', anno.change.revno], clear=1);
-                                   title python:'%s by %s, on %s (%s)' % (anno.change.revno, ', '.join(util.hide_emails(anno.change.authors)), anno.change.date.strftime('%d %b %Y %H:%M'), util.date_time(anno.change.date))"></a>
+                                   title python:'%s by %s, on %s (%s)' % (anno.change.revno, ', '.join(util.hide_emails(anno.change.authors)), anno.change.utc_date.strftime('%d %b %Y %H:%M UTC'), util.date_time(anno.change.utc_date))"></a>
                 <span class="viewAuthors" tal:content="python:'by ' + ', '.join(util.hide_emails(anno.change.authors))" />
                 <br />
 

=== modified file 'loggerhead/util.py'
--- loggerhead/util.py	2011-09-08 00:33:28 +0000
+++ loggerhead/util.py	2012-02-02 07:42:24 +0000
@@ -52,13 +52,12 @@
 # Display of times.
 
 # date_day -- just the day
-# date_time -- full date with time
+# date_time -- full date with time (UTC)
 #
-# displaydate -- for use in sentences
 # approximatedate -- for use in tables
 #
-# displaydate and approximatedate return an elementtree <span> Element
-# with the full date in a tooltip.
+# approximatedate return an elementtree <span> Element
+# with the full date (UTC) in a tooltip.
 
 
 def date_day(value):
@@ -67,19 +66,12 @@
 
 def date_time(value):
     if value is not None:
-        return value.strftime('%Y-%m-%d %H:%M:%S')
+        # Note: this assumes that the value is UTC in some fashion.
+        return value.strftime('%Y-%m-%d %H:%M:%S UTC')
     else:
         return 'N/A'
 
 
-def _displaydate(date):
-    delta = abs(datetime.datetime.now() - date)
-    if delta > datetime.timedelta(1, 0, 0):
-        # far in the past or future, display the date
-        return 'on ' + date_day(date)
-    return _approximatedate(date)
-
-
 def _approximatedate(date):
     delta = datetime.datetime.now() - date
     if abs(delta) > datetime.timedelta(1, 0, 0):
@@ -126,10 +118,6 @@
     return _wrap_with_date_time_title(date, _approximatedate(date))
 
 
-def displaydate(date):
-    return _wrap_with_date_time_title(date, _displaydate(date))
-
-
 class Container(object):
     """
     Convert a dict into an object with attributes.
@@ -226,7 +214,7 @@
 def html_escape(s):
     """Transform dangerous (X)HTML characters into entities.
 
-    Like cgi.escape, except also escaping " and '. This makes it safe to use
+    Like cgi.escape, except also escaping \" and '. This makes it safe to use
     in both attribute and element content.
 
     If you want to safely fill a format string with escaped values, use
@@ -484,7 +472,7 @@
     for index, dir_name in enumerate(dir_parts):
         inner_breadcrumbs.append({
             'dir_name': dir_name,
-            'file_id': inv.path2id('/'.join(dir_parts[:index + 1])),
+            'path': '/'.join(dir_parts[:index + 1]),
             'suffix': '/' + view,
         })
     return inner_breadcrumbs
@@ -555,8 +543,8 @@
 #         for re-ordering an existing page by different sort
 
 t_context = threading.local()
-_valid = ('start_revid', 'file_id', 'filter_file_id', 'q', 'remember',
-          'compare_revid', 'sort')
+_valid = (
+    'start_revid', 'filter_file_id', 'q', 'remember', 'compare_revid', 'sort')
 
 
 def set_context(map):

=== modified file 'setup.py'
--- setup.py	2011-03-10 14:22:12 +0000
+++ setup.py	2012-02-02 04:58:46 +0000
@@ -30,7 +30,10 @@
     license = "GNU GPL v2 or later",
     maintainer = "Michael Hudson",
     maintainer_email = "michael.hudson@canonical.com",
-    scripts = ["serve-branches"],
+    scripts = [
+        "serve-branches",
+        "loggerhead.wsgi",
+        ],
     packages = ["loggerhead",
                 "loggerhead/apps",
                 "loggerhead/controllers",
@@ -65,5 +68,8 @@
                                    "static/images/*"]},
     data_files = [
         ('share/man/man1', ['serve-branches.1']),
+        ('share/doc/loggerhead', ['loggerhead.conf.example',
+                                  'apache-loggerhead.conf',
+                                  'bazaar.conf']),
         ],
     )

